#
# mailest/viewer.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#


module EST


  class Viewer

    def initialize( text, adj )
if $DEBUG
  text.must Gtk::Editable
  adj.must  Gtk::Adjustment
end
      @text = text
      @adj  = adj

      text.set_usize( 640, 200 )
      text.set_editable false

      @port = nil

      @rc = {
        'viewable' => %w( date from subject x-mailer ),
        'hidden'   => %w( received posted message-id  )
      }
    end

    attr :text
    attr :adj
    attr :port


    def view( port )
if $DEBUG then
  port.must ::TMail::Port
end
      @port = port

      unless @port then
        freeze { clear }
        return
      end

      begin
        mail = Mail.new( @port )
        upper = hidden( mail )
        lower = viewable( mail )
      rescue
        upper = ''
        lower = "[Mailest Bug] fail to get mail body\n"
      end

      begin
        freeze {
          clear
          @text.insert_text( upper, 0 )
        }
        val = @adj.upper
        freeze {
          @text.insert_text( "\n", @text.get_length )
          @text.insert_text( lower, @text.get_length )
        }
        @adj.value = val
      rescue
        raise if $DEBUG
        CORE.notice "[EST BUG] viewer: #{$!}"
      end
    end


    def clear
      @text.set_point 0
      @text.forward_delete @text.get_length
    end

    def freeze
      begin
        @text.freeze
        yield
      ensure
        @text.thaw
      end
    end
      
    def read_ahead
      adj = @adj

      if adj.value + adj.page_size >= adj.upper then   # at tail
        false
      else
        # 90% ahead
        adj.value = [ adj.value + adj.page_size * 9 / 10, adj.upper - 1 ].min
        true
      end
    end

    def read_back
      adj = @adj

      if adj.value <= adj.lower then   # at head
        false
      else
        # 90% back
        adj.value = [ adj.value - adj.page_size * 9 / 10, adj.lower ].max
        true
      end
    end



    def hidden( mail )
      rc = @rc['viewable']
      arr = []
      mail.each_header do |k,v|
        arr.push v.decoded unless rc.include? k
      end

      arr.join("\n")
    end

    def viewable( mail )
      ret = @rc['viewable'].collect{|k| mail[k].decoded if mail[k] }
      ret.compact.join("\n") << "\n\n" << mail_to_text( mail ) 
    end

    def mail_to_text( mail )
      ret = nil
      mtype = mail.main_type('').downcase

      case mtype
      when 'text', ''
        ret = mail.body
        case mail.encoding('').downcase
        when '7bit', '8bit', ''
          ;
        when 'base64'
          ret = ret.unpack('m')[0]
        else
          return unknown_encoding( mail )
        end

      when 'multipart'
        mail.parts.each do |m|
          tmp = m.main_type.downcase
          if tmp == 'text' or tmp == '' then
            return mail_to_viewtext(m)
          end
        end
        return unknown_type( mail )

      else
        return unknown_type( mail )
      end

      if NKF.guess( ret ) != NKF::EUC then
        ret = NKF.nkf( '-e', ret )
      end

      # search for "\r\n" from first 200 char
      from = ret.size - 200
      from = from > 0 ? from : 0
      if ret.index( "\r", from ) then
        ret = ret.gsub( "\r\n", "\n" )
      end

      ret
    end

    def undecodable_mail( mail )
      CORE.notice "cannot decode this mail"
      "cannot decode this mail\n" +
      mail['content-transfer-encoding'].decoded
    end

    def unknown_type( mail )
      CORE.notice "cannot display this mail"
      "unknown content type\n" +
      mail['content-type'].decoded
    end

  end


  class List

    def initialize( clist, adj )
      @clist = clist
      @adj = adj
    end


    def inspect
      "#<EST::List @clist=#{@clist.id} @adj=#{@adj.id}>"
    end

    def select( i )
      @clist.select_row i
    end

    def adjust_to( idx )
      return if idx < 0 or idx >= size

      adj = @adj
      val = [ idx * @clist.row_height - adj.page_size / 2, adj.lower ].max
      upp = [ adj.upper - adj.page_size, adj.lower ].max
      adj.value = [ val, upp ].min
      select idx
    end

    def view_next
      adjust_page @selected + 1
    end

    def view_prev
      adjust_page @selected - 1
    end

  end

end   # module EST
